# 单例模式
## 饿汉模式（适合多线程)
* 优缺点说明:
1)优点:这种写法比较简单，就是在类装载的时候就完成实例化。避免了线程同步问题。
2)缺点:在类装载的时候就完成实例化，没有达到Lazy Loading的效果。如果从始
至终从未使用过这个实例,则会造成内存的浪费
* 结论
3)这种方式基于classloder机制避免了多线程的同步问题，不过，instance在类装载
时就实例化，在单例模式中大多数都是调用getInstance方法，但是导致类装载的原因有很多种，因此不能确定有其他的方式（或者其他的静态方法）导致类装载，这时候初始化instance就没有达到lazy loading的效果
4)结论:这种单例模式可用，可能造成内存浪费
``` java 
public class Singleton1 {
    public static void main(String[] args) {
        HungrySingle single1 = HungrySingle.getInstance();
        HungrySingle single2 = HungrySingle.getInstance();
        System.out.println(single1.hashCode());
        System.out.println(single2.hashCode());
    }
}
// 饿汉式(静态变量式)
class HungrySingle {
   // 构造函数私有
    private HungrySingle() {
    }
    private static final HungrySingle hungrySingle = new HungrySingle();
    // 返回instance
    public static HungrySingle getInstance() {
        return hungrySingle;
    }
}
```
## 懒汉模式（适合单线程）
* 说明： 多线程下同时进入代码模块可能对象实例会被创建多次
``` java 
public class LazySingletonOpt {
    public static void main(String[] args) {
        LazySingleton singleton1 = LazySingleton.getInstance();
        LazySingleton singleton2 = LazySingleton.getInstance();
        if(singleton1 == singleton2) {
            
        }
        System.out.println(singleton1.hashCode());
        System.out.println(singleton2.hashCode());
    }
}
// 懒汉式
class LazySingleton {
    private static LazySingleton lazySingleton = null;
    private LazySingleton() {};
    // 判断使用到该方法时再进行创建 lazySingleton
    public static LazySingleton getInstance() {
        if(lazySingleton == null) {
            lazySingleton = new LazySingleton();
        }
        return lazySingleton;
    }

}
```
* 懒汉式为确保多线程安全可加 synchronized 但其实这个方法只执行一次实例化代码就够了，后面的想获得该类实例，直接return就行了。方法进行同步效率太低

